import 'package:tencent_im_sdk_plugin_web/src/enum/message_priority.dart';
import 'package:tencent_im_sdk_plugin_web/src/utils/utils.dart';

class CreateMessage {
  static Object createTextMessage(
      {required String userID,
      required String text,
      String convType = 'C2C',
      int priority = 0}) {
    final textParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({"text": text})
    };
    return mapToJSObj(textParams);
  }

  static Object createCustomMessage(
      {required String userID,
      required String customData,
      int priority = 0,
      String convType = 'C2C',
      String description = '',
      String extension = ''}) {
    final customParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({
        "data": customData,
        "description": description,
        "extension": extension
      })
    };

    return mapToJSObj(customParams);
  }

  static Object createImageMessage(
      {required String userID,
      required file,
      String convType = 'C2C',
      required progressCallback,
      int priority = 0}) {
    final textParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({"file": file}),
      "onProgress": progressCallback
    };
    return mapToJSObj(textParams);
  }

  static Object createVideoMessage(
      {required String userID,
      required file,
      String convType = 'C2C',
      required progressCallback,
      int priority = 0}) {
    final textParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({"file": file}),
      "onProgress": progressCallback
    };
    return mapToJSObj(textParams);
  }

  static Object createFaceMessage(
      {required String userID,
      String convType = 'C2C',
      required int index,
      required data,
      int priority = 0}) {
    final textParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({"index": index, "data": data}),
    };
    return mapToJSObj(textParams);
  }

  static Object createFileMessage(
      {required String userID,
      required file,
      String convType = 'C2C',
      required progressCallback,
      int priority = 0}) {
    final textParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({"file": file}),
      "onProgress": progressCallback
    };
    return mapToJSObj(textParams);
  }

  static Object createTextAtMessage(
      {required String groupID,
      required String text,
      required List<dynamic> atList,
      int priority = 0}) {
    final textParams = {
      "to": groupID,
      "conversationType": "GROUP",
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({"text": text, "atUserList": atList}),
    };
    return mapToJSObj(textParams);
  }

  static Object createLocationMessage(
      {required String userID,
      required String description,
      required double latitude,
      required double longitude,
      String convType = 'C2C',
      int priority = 0}) {
    final textParams = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({
        "description": description,
        "longitude": longitude,
        "latitude": latitude
      }),
    };
    return mapToJSObj(textParams);
  }

  static Object createMereMessage(
      {required String userID,
      required List<dynamic> messageList,
      required String title,
      required String abstractList,
      required String compatibleText,
      String convType = 'C2C',
      int priority = 0}) {
    final params = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": mapToJSObj({
        "messageList": messageList.map((e) => parse(e)),
        "title": title,
        "abstractList": abstractList,
        "compatibleText": compatibleText
      }),
    };

    return mapToJSObj(params);
  }

  static Object createForwardMessage(
      {required String userID,
      required dynamic message,
      String convType = 'C2C',
      int priority = 0}) {
    final params = {
      "to": userID,
      "conversationType": convType,
      "priority": MessagePriorityWeb.convertMsgPriorityToWeb(priority),
      "payload": message,
    };

    return mapToJSObj(params);
  }
}
