import 'dart:convert';

import 'package:tencent_im_sdk_plugin_platform_interface/enum/message_elem_type.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_custom_elem.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_file_elem.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_image.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_image_elem.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_location_elem.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_text_elem.dart';
import 'package:tencent_im_sdk_plugin_platform_interface/models/v2_tim_video_elem.dart';
import 'package:tencent_im_sdk_plugin_web/src/enum/conversation_type.dart';
import 'package:tencent_im_sdk_plugin_web/src/enum/message_type.dart';
import 'package:tencent_im_sdk_plugin_web/src/utils/utils.dart';

class GetConversationList {
  late String conversationID;
  late int count;

  static formateParams(Map<String, dynamic> data) {
    Map<String, dynamic> params = Map<String, dynamic>();
    params["conversationID"] = data["conversationID"];
    params["count"] = data["count"];

    return mapToJSObj(params);
  }

  // getConversationList没有这两个字段
  static Map<String, dynamic> formatReturn(List<dynamic> list) {
    return {
      "nextSeq": "1",
      "isFinished": true,
      "conversationList": list,
    };
  }

  // 将js的数据结构dart化,这里每层为了兼容dart都需要做细致化处理，不然返回回去的某些类型就会报错
  static formateConversationList(dynamic conversationListJs) async {
    final conversationList = [];

    for (var i = 0; i < conversationListJs.length; i++) {
      final item = await formateConversationListItem(conversationListJs[i]);
      conversationList.add(item);
    }

    return conversationList;
  }

  static formateConversationListItem(itemJS) async {
    Map<String, dynamic> conversationListItem = jsToMap(itemJS);

    conversationListItem["friendRemark"] = itemJS.remark;

    if (itemJS.lastMessage != null) {
      conversationListItem["lastMessage"] =
          formateLasteMessage(itemJS.lastMessage);
    }

    // 这个是web特有，其中的属性，在ios都是外层
    if (itemJS.groupProfile != null) {
      conversationListItem["groupProfile"] =
          formateGroupProfile(itemJS.groupProfile);
      conversationListItem["groupID"] = itemJS.groupProfile.groupID;
      conversationListItem["showName"] = getShowName(itemJS);
      conversationListItem["groupType"] = itemJS.groupProfile.type;
      conversationListItem["faceUrl"] = itemJS.groupProfile.avatar;
    }
    if (itemJS.groupAtInfoList != null && itemJS.groupAtInfoList.length > 0) {
      conversationListItem["groupAtInfoList"] =
          formateGroupAtInfoList(itemJS.groupAtInfoList);
    }
    if (itemJS.userProfile != null) {
      conversationListItem["userProfile"] =
          formateUserProfile(itemJS.userProfile);
      conversationListItem["userID"] = itemJS.userProfile.userID;
      conversationListItem["showName"] = getShowName(itemJS);
      conversationListItem["faceUrl"] = itemJS.userProfile.avatar;
    }
    conversationListItem["type"] =
        ConversationTypeWeb.convertConverstationtType(
            conversationListItem["type"]);

    return conversationListItem;
  }

  // TODO 这个lastMessage有点问题，他缺少的东西有点多，比如没有messageID
  static formateLasteMessage(dynamic message) {
    final elementType =
        message.type != null ? MsgType.convertMsgType(message.type) : null;
    // 直接从消息中的Message中copy过来的,但部分字段有缺失
    final formatedMsg = <String, dynamic>{
      "msgID": message.ID ?? '',
      "timestamp": message.lastTime,
      "progress": 100,
      "sender": message.fromAccount,
      "nickName": message.nick,
      "friendRemark": message.remark,
      // "faceUrl": message.avatar ?? '',
      // "nameCard": message.nameCard,
      // "groupID": message.conversationType == "GROUP" ? message.to : null,
      // "userID": message.conversationType == "C2C" ? message.to : null,
      "status": 2, // unSend sucess fail
      "elemType": elementType,
      "localCustomData": "",
      "localCustomInt": 0,
      "cloudCustomData": message.cloudCustomData,
      "isSelf": true,
      "isRead": message.isRead,
      // "isPeerRead": message.isPeerRead > 0 ? true : false,
      // "priority": convertPriorityFromWebToDart(message.priority),
      // "groupAtUserList": message.atUserList,
      // "random": message.random,
      "isExcludedFromUnreadCount": false,
      // "messageFromWeb": stringify(message)
    };

    // 文本消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_TEXT) {
      final textElem = V2TimTextElem(text: message.payload.text).toJson();
      formatedMsg['textElem'] = textElem;
    }

    // 自定义消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_CUSTOM) {
      final customElem = V2TimCustomElem(
              data: message.payload.data,
              desc: message.payload.discription,
              extension: message.payload.extension)
          .toJson();
      formatedMsg['customElem'] = customElem;
    }

    // 图片消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_IMAGE) {
      final imageInfoArray = message.payload.imageInfoArray as List;
      final imagePath = imageInfoArray[0].url;
      final UUID = message.payload.uuid;
      final imageList = imageInfoArray
          .map((e) => V2TimImage(
              type: e.type,
              height: e.height,
              width: e.width,
              size: e.size,
              UUID: UUID,
              url: e.imageUrl))
          .toList();
      final imageElem = V2TimImageElem(path: imagePath, imageList: imageList);
      formatedMsg['imageElem'] = imageElem.toJson();
    }

    //视频消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_VIDEO) {
      final messagePayload = message.payload;
      final videoElem = V2TimVideoElem(
          videoPath: messagePayload.videoUrl,
          duration: messagePayload.videoSecond,
          UUID: messagePayload.videoUUID,
          snapshotPath: messagePayload.thumbUrl,
          snapshotUUID: messagePayload.thumbUUID,
          snapshotSize: messagePayload.thumbSize,
          snapshotWidth: messagePayload.thumbWidth,
          snapshotHeight: messagePayload.thumbHeight,
          videoUrl: messagePayload.videoUrl,
          videoSize: messagePayload.videoSize);
      formatedMsg['videoElem'] = videoElem.toJson();
    }

    // 文件消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_FILE) {
      final messagePayload = message.payload;
      final fileElem = V2TimFileElem(
              path: messagePayload.fileUrl,
              fileName: messagePayload.fileName,
              UUID: messagePayload.uuid,
              fileSize: messagePayload.fileSize)
          .toJson();
      formatedMsg['fileElem'] = fileElem;
    }

    // 地理位置消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_LOCATION) {
      final locationElem = V2TimLocationElem(
          desc: message.payload.description,
          longitude: message.payload.longitude,
          latitude: message.payload.latitude);
      formatedMsg['locationElem'] = locationElem.toJson();
    }

    // 合并消息
    if (elementType == MessageElemType.V2TIM_ELEM_TYPE_MERGER) {}

    return formatedMsg;

    // return await V2TIMMessage.convertMessageFromWebToDart(itemJS); // 不能直接使用message的处理函数
  }

  // TODO 暂时未使用
  static getConversationListLastMessage(message) {
    // lasteMessage is C2C
    final lastMessageJS = message.lastMessage;
    Map<String, dynamic> formatedMsg = formateLasteMessage(lastMessageJS);
    if (message.userProfile != null) {
      formatedMsg["faceUrl"] = message.userProfile.avatar;
      formatedMsg["nickName"] = message.userProfile.nick;
      formatedMsg["userID"] = message.userProfile.userID;
    }

    if (message.groupProfile != null) {
      formatedMsg["groupID"] = message.groupProfile.groupID;
    }
    // lasteMessage is Group
    if (message.groupProfile != null) {
      formatedMsg["groupID"] = message.groupProfile.groupID;
    }
  }

  // 群组即群名称
  // C2C： 好友备注 => 昵称 =》 userID
  static String getShowName(itemJS) {
    if (itemJS.remark != null) {
      return itemJS.remark;
    } else if (itemJS.groupProfile != null) {
      return itemJS.groupProfile.name;
    } else {
      return itemJS.userProfile.nickName ?? itemJS.userProfile.userID;
    }
  }

  static formateGroupAtInfoList(dynamic groupAtInfoListJs) {
    print("formateGroupAtInfoList");
    final groupAtInfoList = [];
    for (var i = 0; i < groupAtInfoListJs.length; i++) {
      final item = {
        "seq": (groupAtInfoListJs[i].messageSequence).toString(),
        "atType": groupAtInfoListJs[i].atTypeArray[0] //TODO 这里返回的是一个Array我不懂了
      };
      groupAtInfoList.add(item);
    }
    return groupAtInfoList;
  }

  static formateGroupProfile(dynamic groupProfileJS) {
    // print("formateGroupProfile");
    var groupProfile = jsToMap(groupProfileJS);
    if (groupProfileJS.lastMessage != null) {
      groupProfile["lastMessage"] =
          formateLasteMessage(groupProfileJS.lastMessage);
    }
    if (groupProfileJS.selfInfo != null) {
      groupProfile["selfInfo"] = jsToMap(groupProfileJS.selfInfo);
    }
    if (groupProfileJS.groupCustomField != null &&
        groupProfileJS.groupCustomField.length > 0) {
      groupProfile["groupCustomField"] =
          formateGroupCustomField(groupProfileJS.groupCustomField);
    }
    // print("formateGroupProfile");
    return groupProfile;
  }

  static formateProfileCustomField(dynamic profileCustomFieldJS) {
    final groupCustomField = [];
    for (var i = 0; i < profileCustomFieldJS.length; i++) {
      final item = (profileCustomFieldJS[i]);
      groupCustomField.add(item);
    }

    return groupCustomField;
  }

  static formateGroupCustomField(dynamic groupCustomFieldJs) {
    final groupCustomField = [];
    for (var i = 0; i < groupCustomFieldJs.length; i++) {
      final item = jsToMap(groupCustomFieldJs[i]);
      groupCustomField.add(item);
    }

    return groupCustomField;
  }

  static formateUserProfile(dynamic userProfileJS) {
    var userProfile = jsToMap(userProfileJS);
    if (userProfileJS.profileCustomField != null &&
        userProfileJS.profileCustomField.length > 0) {
      userProfile["profileCustomField"] =
          formateProfileCustomField(userProfileJS.profileCustomField);
    }
    return userProfile;
  }
}
